// $Id: advance.C,v 1.1 2001/08/14 07:42:41 dvermeir Exp $

// Illustrate advance algorithm which dispatches on the iterator
// category.

#include	<assert.h>
#include	<iostream>

#include	"advance.h"

// Node* is a (too) simple linked list.
template <typename T>
class Node { 
public:
  Node(const T& t): value_(t), cdr_(0) {} // you cannot make an empty list
  Node*	cons(const T& t) { return new Node(t,this); } // prepend
  const T&	car() const { return value_; } // non const version missing
  Node*	cdr() const { return cdr_; } // non const version missing
  
  ///////////////////////////////////////////////////////////////////
  // Iterator class type satisfying requirements for find.
  class Cursor { 
  public:
    // Iterator_traits<Node::Cursor> will refer to these.

    // Value_type refers to the type underlying *Cursor.
    typedef T 			 value_type;
    typedef T* 			 pointer;
    typedef T& 			 reference;
    typedef ptrdiff_t		 difference_type;
    typedef forward_iterator_tag iterator_category;

    Cursor(Node* node=0): node_(node) {}
    const value_type& operator*() const { assert(node_); return node_->car(); }
    Cursor& operator++() { 
      assert(node_); 
      node_ = node_->cdr(); 
      return *this; 
    }
    Cursor operator++(int) { 
      Cursor tmp(*this); 
      ++*this; 
      return tmp; 
      }
    bool operator!=(const Cursor& c) const { 
      return node_ != c.node_; }
    bool operator==(const Cursor& c) const { 
      return node_ == c.node_; }
  private:
    Node*	node_;
  };
  //
  ///////////////////////////////////////////////////////////////////

private:
  Node(const T& t,Node* cdr): value_(t), cdr_(cdr) {}
  T		value_;
  Node*		cdr_;
};

int
main() {

// Advance with linked list (forward iterator).

Node<int>*	list(new Node<int>(99));

for (int i=0; i<99; ++i)
  list = list->cons(i);

Node<int>::Cursor it1(list);

myadvance(it1,50);

cout << *it1 << endl;

// Advance with array (random access iterator).

int	a[100];
for (int i=0; i<100; ++i)
  a[i] = i;

int* it2(a);
myadvance(it2,50);

cout << *it2 << endl;

}
