#ifndef DICT_H
#define DICT_H
// $Id: dict.h,v 1.1 2001/08/14 08:24:28 dvermeir Exp $

#include	<string>
#include	<vector>
#include	<list>
#include	<algorithm>

// Simple scramble function that computes an int from a string.
inline unsigned int
scramble(string s) {
unsigned int result=1;
for (unsigned int i=0;(i<s.size());++i)
  result *= s[i];
return result;
}

// A class that keeps a dictionary of words using a hash table.
// It uses hashing with separate chaining: the primary table
// is a vector of lists.
class Dictionary {
public:
  Dictionary(unsigned int size): size_(size), words_(size) {}
  // Does the word appear in the dictionary?
  bool find(const string& s) { 
    list<string>&	l(words_[hash(s)]);
    return ::find(l.begin(),l.end(),s)!=l.end();
    }
  // Insert a new word in the dictionary.
  void insert(const string& s) { if (!find(s)) words_[hash(s)].push_back(s); }
  // (division) hash function.
  unsigned int hash(const string& s) { return scramble(s)%size_; }
private:
  unsigned int		size_;
  vector<list<string> >	words_;
};
#endif
